﻿using UnityEngine;
using System.Collections;

using System.Text;
using System.IO;

/*
*   エディタで作成されたStageDataファイルを読み込んで
*   配列に対応付けされたPrefabにコンバート、配置する。
*
*   配置後、任意でカメラ位置をすべての配置したタイルが収まるように調整する
*/

public class StageGenerator : MonoBehaviour
{
    [SerializeField] int _stageNumber = 0;

    [Tooltip("XZ平面上での1チップの大きさ")]
    [SerializeField] int _tileSize = 5;

    [SerializeField] bool _adjustCameraPosition;
    [SerializeField] Camera _mainCamera;

    [Tooltip("マップエディタの数値に対応する各GameObjectのPrefab")]
    [SerializeField] GameObject[] _prefabs;

    [Tooltip("カメラをどれくらいマップから離すかの倍率")]
    [SerializeField] float _distanceRatio = 1f;

    [Tooltip("タイルを子オブジェクトとして統合するオブジェクト")]
    [SerializeField] Transform _tilesHolder;

    [Tooltip("外壁に設定するPrefab")]
    [SerializeField] GameObject _outerWallPrefab;

    public int RowNumber
    {
            get { return _rowNumber; }
    }
    public int ColumnNumber
    {
        get { return _columnNumber; }
    }
    public int TileSize
    {
        get { return _tileSize; }
    }

    public Vector3 GetUpperLeft()
    {
        return _upperLeft.transform.position - _upperLeft.transform.localScale / 2f;
    }

    int _rowNumber = 0;
    int _columnNumber = 0;

    GameObject _upperLeft;

    // Use this for initialization
    void Start()
    {
        _stageNumber = BaseScripts.getInstance().generateStageNumber;

        ReadStage();

        //カメラ位置の調整の必要が無ければ終了
        if (!_adjustCameraPosition)
            return;

        if (_mainCamera == null)
            _mainCamera = GetComponent<Camera>();

        if (_mainCamera == null)
        {
            Debug.LogWarning("位置を調整する対象となるカメラが設定されていません");
            return;
        }

        AdjustCameraPosition();

        CreateOuterWall();
    }

    // Update is called once per frame
    void Update()
    {
        //カメラ位置調整がONなら、位置調整
        if (_mainCamera != null && _adjustCameraPosition)
            AdjustCameraPosition();
    }

    void ReadStage()
    {
        //ステージcsvを読み込む
        StreamReader reader = new StreamReader(Application.dataPath + "/StageData/Stage" + _stageNumber.ToString() + ".txt");

        while (!reader.EndOfStream)
        {
            _rowNumber = 0;

            //列ごと読み込んで
            string line = reader.ReadLine();

            //1タイルごとに分解
            string[] elements = line.Split(',');

            //列の文字からそれぞれPrefab化
            foreach (var elem in elements)
            {
                GeneratePrefab(elem, _rowNumber, _columnNumber);
                _rowNumber++;
            }

            //改行処理
            _columnNumber++;
        }
    }

    void GeneratePrefab(string element, int rowNumber, int columnNumber)
    {
        var obj = Instantiate(ConvertCSVToPrefab(element));

        obj.transform.position = new Vector3(rowNumber * _tileSize,
                                             0,
                                             -columnNumber * _tileSize);

        if (_tilesHolder != null)
            obj.transform.parent = _tilesHolder.transform;

        //タイルの基準とすべく左上のオブジェクトをキャッシュ
        if (    rowNumber == 0
            && _columnNumber == 0)
        {
            _upperLeft = obj;
        }
    }

    GameObject ConvertCSVToPrefab(string element)
    {
        //読み込んだ文字列を変換
        int elementNumber = int.Parse(element);

        //範囲外の値が出たら、デフォルトのPrefabを返す
        if (elementNumber < 0
            || _prefabs.Length < elementNumber)
        {
            return _prefabs[0];
        }

        return _prefabs[elementNumber];
    }

    void AdjustCameraPosition()
    {
         AdjustCameraOnOrthoGraphic();
    }
    
    void AdjustCameraOnOrthoGraphic()
    {
        float ratio = _tileSize * Mathf.Max(_columnNumber, _rowNumber);

        //ステージがはまるように
        _mainCamera.orthographicSize = ratio * 0.4f;
        _mainCamera.transform.position = new Vector3(0.8f, 0.5f, -1.1f) * ratio;

        _mainCamera.orthographicSize += ratio * 0.05f;
        _mainCamera.transform.position += new Vector3(0f, 0.1f/0.6f, 0f) * ratio;
    }

    void CreateOuterWall()
    {
        if (_outerWallPrefab == null)
            Debug.LogWarning("OuterWall Is not setting");

        _outerWallPrefab.transform.localScale = new Vector3(1, 5, 20 * Mathf.Max(_columnNumber, _rowNumber));

        var leftWall = Instantiate(_outerWallPrefab);
        leftWall.transform.position = new Vector3(0f, 0, 1f);

        var rightWall = Instantiate(_outerWallPrefab);
        rightWall.transform.position = new Vector3(0f, 0f, -_rowNumber);

        var forwardWall = Instantiate(_outerWallPrefab);
        forwardWall.transform.position = new Vector3(_columnNumber, 0, 0);
        forwardWall.transform.Rotate(Vector3.up, 90f);

        var backWall = Instantiate(_outerWallPrefab);
        backWall.transform.position = new Vector3(-1, 0, 0);
        backWall.transform.Rotate(Vector3.up, 90f);
    }
}
